package org.msrb.emma.client.domain.primarymarket;

import com.google.gson.Gson;
import com.google.gson.GsonBuilder;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.IOException;
import javax.xml.bind.JAXBContext;
import org.junit.Test;
import org.msrb.emma.client.domain.continuingdisclosure.ContinuingDisclosureSubmissionDetailsType;
import org.msrb.emma.client.domain.financialreporting.FinancialReportingSubmissionDetailsType;
import org.msrb.emma.client.domain.shortdocs.ShortSubmissionDetailsType;

/**
 *
 * @author admin
 */
public class DeserializationModelTest {
	
	@Test
	public void shortDocSubmissionType() throws Exception {
		final JAXBContext context = JAXBContext.newInstance(ShortSubmissionDetailsType.class);
		final ShortSubmissionDetailsType submission = (ShortSubmissionDetailsType) context.createUnmarshaller().unmarshal(getSubmissionText("ShortDocSubmission"));
		outputResult(convertToJson(new GenericSubmissionWrapper(submission)), "ShortDocSubmission");
	}
	
	@Test
	public void financialReportingSubmissionType() throws Exception {
		final JAXBContext context = JAXBContext.newInstance(FinancialReportingSubmissionDetailsType.class);
		final FinancialReportingSubmissionDetailsType submission = (FinancialReportingSubmissionDetailsType) context.createUnmarshaller().unmarshal(getSubmissionText("FinancialReportingSubmission"));
		outputResult(convertToJson(new FinancialReportingWrapper(submission)), "FinancialReportingSubmission");
	}
	
	@Test
	public void continuingDisclosureSubmissionType() throws Exception {
		final JAXBContext context = JAXBContext.newInstance(ContinuingDisclosureSubmissionDetailsType.class);
		final ContinuingDisclosureSubmissionDetailsType submission = (ContinuingDisclosureSubmissionDetailsType) context.createUnmarshaller().unmarshal(getSubmissionText("ContinuingDisclosureSubmissionType"));
		outputResult(convertToJson(new ContinuingDisclosureWrapper(submission)), "ContinuingDisclosureSubmissionType");
	}
	
	@Test
	public void submissionDetailsType() throws Exception {
		final JAXBContext context = JAXBContext.newInstance(SubmissionDetailsType.class);
		final SubmissionDetailsType submission = (SubmissionDetailsType) context.createUnmarshaller().unmarshal(getSubmissionText("SubmissionDetailsType"));
		outputResult(convertToJson(new GenericSubmissionWrapper(submission)), "SubmissionDetailsType");
	}
	
	private static String convertToJson(Object submission) {
		Gson converter = new GsonBuilder()
				.setPrettyPrinting()
				.create();
		return converter.toJson(submission);
	}

	private FileReader getSubmissionText(String fileName) throws FileNotFoundException {
		return new FileReader(fileName + ".xml");
	}

	private static void outputResult(String text, String fileName) throws IOException {
		try (FileWriter fileWriter = new FileWriter(fileName + ".json")) {
			fileWriter.write(text);
		}
	}
	
	private static class GenericSubmissionWrapper {
		private final Object submission;

		public GenericSubmissionWrapper(Object submission) {
			this.submission = submission;
		}
		
		public Object getSubmission() {
			return submission;
		}
	}
	
	private static class ContinuingDisclosureWrapper {
		private final Object submission;

		public ContinuingDisclosureWrapper(Object submission) {
			this.submission = submission;
		}
		
		public Object getContinuingDisclosureSubmission() {
			return submission;
		}
	}
	
	private static class FinancialReportingWrapper {
		private final Object submission;

		public FinancialReportingWrapper(Object submission) {
			this.submission = submission;
		}
		
		public Object getFinancialReportingSubmission() {
			return submission;
		}
	}
	
}
